@@ -1,186 +1,254 @@
 /* Replacements for routines missing on some systems.
    Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
-   
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+This file is part of Wget.
 
-/* $Id: cmpt.c,v 1.1.1.1.2.2 1997/02/17 06:19:58 hniksic Exp $ */
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif /* HAVE_CONFIG_H */
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_STRING_H
-#  include <string.h>
+# include <string.h>
 #else
-#  include <strings.h>
+# include <strings.h>
 #endif /* HAVE_STRING_H */
 #include <ctype.h>
 
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
-#  include <unistd.h>
+# include <unistd.h>
 #endif
 #include <limits.h>
 
 #include "wget.h"
-#include "cmpt.h"
 
-
-/* Define strdup for systems that don't have it.  */
-#ifndef HAVE_STRDUP
+#ifndef HAVE_STRERROR
+/* A strerror() clone, for systems that don't have it.  */
 char *
-strdup(const char *s)
+strerror (int err)
 {
-   char *s1;
-   size_t l;
-
-   l = strlen(s);
-   s1 = malloc(l + 1);
-   if (!s1)
-      return NULL;
-   memcpy(s1, s, l + 1);
-   return s1;
+  /* This loses on a system without `sys_errlist'.  */
+  extern char *sys_errlist[];
+  return sys_errlist[err];
 }
-#endif
-
+#endif /* not HAVE_STRERROR */
 
 /* Some systems don't have some str* functions in libc.  Here we
    define them.  The same goes for strptime.  */
 
 #ifndef HAVE_STRCASECMP
-/* From GNU libc. */
+/* From GNU libc.  */
 /* Compare S1 and S2, ignoring case, returning less than, equal to or
    greater than zero if S1 is lexiographically less than,
    equal to or greater than S2.  */
 int
-strcasecmp(const char *s1, const char *s2)
+strcasecmp (const char *s1, const char *s2)
 {
-   register const unsigned char *p1 = (const unsigned char *) s1;
-   register const unsigned char *p2 = (const unsigned char *) s2;
-   unsigned char c1, c2;
+  register const unsigned char *p1 = (const unsigned char *) s1;
+  register const unsigned char *p2 = (const unsigned char *) s2;
+  unsigned char c1, c2;
 
-   if (p1 == p2)
-      return 0;
+  if (p1 == p2)
+    return 0;
 
-   do
-   {
+  do
+    {
       c1 = tolower (*p1++);
       c2 = tolower (*p2++);
       if (c1 == '\0')
-	 break;
-   }
-   while (c1 == c2);
+	break;
+    }
+  while (c1 == c2);
 
-   return c1 - c2;
+  return c1 - c2;
 }
-#endif /* HAVE_STRCASECMP */
+#endif /* not HAVE_STRCASECMP */
 
 #ifndef HAVE_STRNCASECMP
-/* From GNU libc. */
+/* From GNU libc.  */
 /* Compare no more than N characters of S1 and S2,
    ignoring case, returning less than, equal to or
    greater than zero if S1 is lexicographically less
    than, equal to or greater than S2.  */
 int
-strncasecmp(const char *s1, const char *s2, size_t n)
+strncasecmp (const char *s1, const char *s2, size_t n)
 {
-   register const unsigned char *p1 = (const unsigned char *) s1;
-   register const unsigned char *p2 = (const unsigned char *) s2;
-   unsigned char c1, c2;
+  register const unsigned char *p1 = (const unsigned char *) s1;
+  register const unsigned char *p2 = (const unsigned char *) s2;
+  unsigned char c1, c2;
 
-   if (p1 == p2 || n == 0)
-      return 0;
+  if (p1 == p2 || n == 0)
+    return 0;
 
-   do
-   {
+  do
+    {
       c1 = tolower (*p1++);
       c2 = tolower (*p2++);
       if (c1 == '\0' || c1 != c2)
-	 return c1 - c2;
-   } while (--n > 0);
+	return c1 - c2;
+    } while (--n > 0);
 
-   return c1 - c2;
+  return c1 - c2;
 }
-#endif /* HAVE_STRNCASECMP */
+#endif /* not HAVE_STRNCASECMP */
 
 #ifndef HAVE_STRSTR
-/* From GNU libc. */
+/* From GNU libc 2.0.6.  */
 /* Return the first ocurrence of NEEDLE in HAYSTACK.  */
+/*
+ * My personal strstr() implementation that beats most other algorithms.
+ * Until someone tells me otherwise, I assume that this is the
+ * fastest implementation of strstr() in C.
+ * I deliberately chose not to comment it.  You should have at least
+ * as much fun trying to understand it, as I had to write it :-).
+ *
+ * Stephen R. van den Berg, berg@pool.informatik.rwth-aachen.de	*/
+typedef unsigned chartype;
+
 char *
-strstr(const char *const haystack, const char *const needle)
+strstr (phaystack, pneedle)
+     const char *phaystack;
+     const char *pneedle;
 {
-   register const char *const needle_end = strchr(needle, '\0');
-   register const char *const haystack_end = strchr(haystack, '\0');
-   register const size_t needle_len = needle_end - needle;
-   register const size_t needle_last = needle_len - 1;
-   register const char *begin;
+  register const unsigned char *haystack, *needle;
+  register chartype b, c;
 
-   if (needle_len == 0)
-      return (char *) haystack;	/* ANSI 4.11.5.7, line 25.  */
-   if ((size_t) (haystack_end - haystack) < needle_len)
-      return NULL;
-
-   for (begin = &haystack[needle_last]; begin < haystack_end; ++begin)
-   {
-      register const char *n = &needle[needle_last];
-      register const char *h = begin;
+  haystack = (const unsigned char *) phaystack;
+  needle = (const unsigned char *) pneedle;
 
+  b = *needle;
+  if (b != '\0')
+    {
+      haystack--;				/* possible ANSI violation */
       do
-	 if (*h != *n)
-	    goto loop;		/* continue for loop */
-      while (--n >= needle && --h >= haystack);
+	{
+	  c = *++haystack;
+	  if (c == '\0')
+	    goto ret0;
+	}
+      while (c != b);
 
-      return (char *) h;
+      c = *++needle;
+      if (c == '\0')
+	goto foundneedle;
+      ++needle;
+      goto jin;
 
-     loop:;
-   }
+      for (;;)
+        {
+          register chartype a;
+	  register const unsigned char *rhaystack, *rneedle;
 
-   return NULL;
+	  do
+	    {
+	      a = *++haystack;
+	      if (a == '\0')
+		goto ret0;
+	      if (a == b)
+		break;
+	      a = *++haystack;
+	      if (a == '\0')
+		goto ret0;
+shloop:	    }
+          while (a != b);
+
+jin:	  a = *++haystack;
+	  if (a == '\0')
+	    goto ret0;
+
+	  if (a != c)
+	    goto shloop;
+
+	  rhaystack = haystack-- + 1;
+	  rneedle = needle;
+	  a = *rneedle;
+
+	  if (*rhaystack == a)
+	    do
+	      {
+		if (a == '\0')
+		  goto foundneedle;
+		++rhaystack;
+		a = *++needle;
+		if (*rhaystack != a)
+		  break;
+		if (a == '\0')
+		  goto foundneedle;
+		++rhaystack;
+		a = *++needle;
+	      }
+	    while (*rhaystack == a);
+
+	  needle = rneedle;		/* took the register-poor approach */
+
+	  if (a == '\0')
+	    break;
+        }
+    }
+foundneedle:
+  return (char*) haystack;
+ret0:
+  return 0;
 }
-#endif /* HAVE_STRSTR */
+#endif /* not HAVE_STRSTR */
 
 #ifndef HAVE_MKTIME
-/* From GNU libc 2.0. */
+/* From GNU libc 2.0.  */
+
+/* Copyright (C) 1993, 1994, 1995, 1996, 1997 Free Software Foundation, Inc.
+   This file is part of the GNU C Library.
+   Contributed by Paul Eggert (eggert@twinsun.com).  */
+
+#ifdef _LIBC
+# define HAVE_LIMITS_H 1
+# define HAVE_LOCALTIME_R 1
+# define STDC_HEADERS 1
+#endif
 
 /* Assume that leap seconds are possible, unless told otherwise.
    If the host has a `zic' command with a `-L leapsecondfilename' option,
    then it supports leap seconds; otherwise it probably doesn't.  */
 #ifndef LEAP_SECONDS_POSSIBLE
-#define LEAP_SECONDS_POSSIBLE 1
+# define LEAP_SECONDS_POSSIBLE 1
 #endif
 
+#ifndef __P
+# define __P(args) PARAMS (args)
+#endif  /* Not __P.  */
+
 #ifndef CHAR_BIT
-#define CHAR_BIT 8
+# define CHAR_BIT 8
 #endif
 
 #ifndef INT_MIN
-#define INT_MIN (~0 << (sizeof (int) * CHAR_BIT - 1))
+# define INT_MIN (~0 << (sizeof (int) * CHAR_BIT - 1))
 #endif
 #ifndef INT_MAX
-#define INT_MAX (~0 - INT_MIN)
+# define INT_MAX (~0 - INT_MIN)
 #endif
 
 #ifndef TIME_T_MIN
-#define TIME_T_MIN (0 < (time_t) -1 ? (time_t) 0 \
-		    : ~ (time_t) 0 << (sizeof (time_t) * CHAR_BIT - 1))
+/* The outer cast to time_t works around a bug in Cray C 5.0.3.0.  */
+# define TIME_T_MIN ((time_t) \
+		    (0 < (time_t) -1 ? (time_t) 0 \
+		     : ~ (time_t) 0 << (sizeof (time_t) * CHAR_BIT - 1)))
 #endif
 #ifndef TIME_T_MAX
-#define TIME_T_MAX (~ (time_t) 0 - TIME_T_MIN)
+# define TIME_T_MAX (~ (time_t) 0 - TIME_T_MIN)
 #endif
 
 #define TM_YEAR_BASE 1900
@@ -189,7 +257,7 @@ strstr(const char *const haystack, const char *const needle)
 #ifndef __isleap
 /* Nonzero if YEAR is a leap year (every 4 years,
    except every 100th isn't, and every 400th is).  */
-#define	__isleap(year)	\
+# define __isleap(year)	\
   ((year) % 4 == 0 && ((year) % 100 != 0 || (year) % 400 == 0))
 #endif
 
@@ -202,19 +270,19 @@ const unsigned short int __mon_yday[2][13] =
     { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }
   };
 
-static time_t ydhms_tm_diff PARAMS ((int, int, int, int, int, const struct tm *));
-time_t __mktime_internal PARAMS ((struct tm *,
+static time_t ydhms_tm_diff __P ((int, int, int, int, int, const struct tm *));
+time_t __mktime_internal __P ((struct tm *,
 			       struct tm *(*) (const time_t *, struct tm *),
 			       time_t *));
 
 
-#if ! HAVE_LOCALTIME_R && ! defined (localtime_r)
 #ifdef _LIBC
-#define localtime_r __localtime_r
+# define localtime_r __localtime_r
 #else
+# if ! HAVE_LOCALTIME_R && ! defined localtime_r
 /* Approximate localtime_r as best we can in its absence.  */
-#define localtime_r my_localtime_r
-static struct tm *localtime_r PARAMS ((const time_t *, struct tm *));
+#  define localtime_r my_mktime_localtime_r
+static struct tm *localtime_r __P ((const time_t *, struct tm *));
 static struct tm *
 localtime_r (t, tp)
      const time_t *t;
@@ -226,8 +294,8 @@ localtime_r (t, tp)
   *tp = *l;
   return tp;
 }
+# endif /* ! HAVE_LOCALTIME_R && ! defined (localtime_r) */
 #endif /* ! _LIBC */
-#endif /* ! HAVE_LOCALTIME_R && ! defined (localtime_r) */
 
 
 /* Yield the difference between (YEAR-YDAY HOUR:MIN:SEC) and (*TP),
@@ -286,7 +354,7 @@ mktime (tp)
 time_t
 __mktime_internal (tp, convert, offset)
      struct tm *tp;
-     struct tm *(*convert) PARAMS ((const time_t *, struct tm *));
+     struct tm *(*convert) __P ((const time_t *, struct tm *));
      time_t *offset;
 {
   time_t t, dt, t0;
@@ -325,10 +393,10 @@ __mktime_internal (tp, convert, offset)
 	       [mon_remainder + 12 * negative_mon_remainder])
 	      + mday - 1);
 
+  int sec_requested = sec;
 #if LEAP_SECONDS_POSSIBLE
   /* Handle out-of-range seconds specially,
      since ydhms_tm_diff assumes every minute has 60 seconds.  */
-  int sec_requested = sec;
   if (sec < 0)
     sec = 0;
   if (59 < sec)
@@ -410,12 +478,12 @@ __mktime_internal (tp, convert, offset)
 #ifdef weak_alias
 weak_alias (mktime, timelocal)
 #endif
-#endif /* HAVE_MKTIME */
+#endif /* not HAVE_MKTIME */
 
 
 #ifndef HAVE_STRPTIME
-/* From GNU libc 2.0, by Ulrich Drepper <drepper@gnu.ai.mit.edu>.  */
-/* Ulrich, thanks for helping me!  */
+/* From GNU libc 2.0.6.  */
+/* Ulrich, thanks for helping me out with this!  --hniksic  */
 
 /* strptime - Convert a string representation of time to a time value.
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
@@ -427,6 +495,9 @@ weak_alias (mktime, timelocal)
    some of them in the same format (such as year, week and weekday)
    this is enough information for determining the date.  */
 
+#ifndef __P
+# define __P(args) PARAMS (args)
+#endif /* not __P */
 
 #if ! HAVE_LOCALTIME_R && ! defined (localtime_r)
 #ifdef _LIBC
@@ -434,8 +505,11 @@ weak_alias (mktime, timelocal)
 #else
 /* Approximate localtime_r as best we can in its absence.  */
 #define localtime_r my_localtime_r
+static struct tm *localtime_r __P ((const time_t *, struct tm *));
 static struct tm *
-localtime_r (const time_t *t, struct tm *tp)
+localtime_r (t, tp)
+     const time_t *t;
+     struct tm *tp;
 {
   struct tm *l = localtime (t);
   if (! l)
@@ -558,10 +632,16 @@ static char const ab_month_name[][4] =
 /* Status of lookup: do we use the locale data or the raw data?  */
 enum locale_status { not, loc, raw };
 
+static char *
+strptime_internal __P ((const char *buf, const char *format, struct tm *tm,
+			enum locale_status *decided));
 
 static char *
-strptime_internal (const char *buf, const char *format, struct tm *tm,
-		   enum locale_status *decided)
+strptime_internal (buf, format, tm, decided)
+     const char *buf;
+     const char *format;
+     struct tm *tm;
+     enum locale_status *decided;
 {
   const char *rp;
   const char *fmt;
@@ -750,7 +830,7 @@ strptime_internal (const char *buf, const char *format, struct tm *tm,
 	case 'I':
 	  /* Match hour in 12-hour clock.  */
 	  get_number (1, 12);
-	  tm->tm_hour = val - 1;
+	  tm->tm_hour = val % 12;
 	  have_I = 1;
 	  break;
 	case 'j':
@@ -910,11 +990,14 @@ strptime_internal (const char *buf, const char *format, struct tm *tm,
 	case 'y':
 	  /* Match year within century.  */
 	  get_number (0, 99);
-	  tm->tm_year = val;
+	  tm->tm_year = val >= 50 ? val : val + 100;
 	  break;
 	case 'Y':
 	  /* Match year including century number.  */
-	  get_number (0, INT_MAX);
+	  if (sizeof (time_t) > 4)
+	    get_number (0, 9999);
+	  else
+	    get_number (0, 2036);
 	  tm->tm_year = val - 1900;
 	  break;
 	case 'Z':
@@ -1092,7 +1175,10 @@ strptime_internal (const char *buf, const char *format, struct tm *tm,
 
 
 char *
-strptime (const char *buf, const char *format, struct tm *tm)
+strptime (buf, format, tm)
+     const char *buf;
+     const char *format;
+     struct tm *tm;
 {
   enum locale_status decided;
 #ifdef _NL_CURRENT
@@ -1102,6 +1188,4 @@ strptime (const char *buf, const char *format, struct tm *tm)
 #endif
   return strptime_internal (buf, format, tm, &decided);
 }
-
-#endif /* HAVE_STRPTIME */
-
+#endif /* not HAVE_STRPTIME */
