@@ -21,15 +21,14 @@
    Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
-/* $Id: getopt.c,v 1.1.1.1.2.1 1997/02/15 19:22:55 hniksic Exp $ */
-
 
 /* NOTE!!!  AIX requires this to be the first thing in the file.
    Do not put ANYTHING before it!  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+# include <config.h>
 #endif /* HAVE_CONFIG_H */
+#include "wget.h"
 
 #if !__STDC__ && !defined(const) && IN_GCC
 #define const
@@ -42,9 +41,9 @@
 
 #include <stdio.h>
 #ifdef HAVE_STRING_H
-#  include <string.h>
+# include <string.h>
 #else
-#  include <strings.h>
+# include <strings.h>
 #endif
 
 /* Comment out all this code if we are using the GNU C Library, and are not
@@ -174,9 +173,7 @@ static enum
 char *getenv ();
 
 static char *
-my_index (str, chr)
-     const char *str;
-     int chr;
+my_index (const char *str, int chr)
 {
   while (*str)
     {
@@ -188,10 +185,7 @@ my_index (str, chr)
 }
 
 static void
-my_bcopy (from, to, size)
-     const char *from;
-     char *to;
-     int size;
+my_bcopy (const char *from, char *to, int size)
 {
   int i;
   for (i = 0; i < size; i++)
@@ -218,11 +212,10 @@ static int last_nonopt;
    the new indices of the non-options in ARGV after they are moved.  */
 
 static void
-exchange (argv)
-     char **argv;
+exchange (char **argv)
 {
   int nonopts_size = (last_nonopt - first_nonopt) * sizeof (char *);
-  char **temp = (char **) malloc (nonopts_size);
+  char **temp = (char **) alloca (nonopts_size);
 
   /* Interchange the two blocks of data in ARGV.  */
 
@@ -237,8 +230,6 @@ exchange (argv)
 
   first_nonopt += (optind - last_nonopt);
   last_nonopt = optind;
-
-  free(temp);
 }
 
 /* Scan elements of ARGV (whose length is ARGC) for option characters
@@ -298,13 +289,8 @@ exchange (argv)
    long-named options.  */
 
 int
-_getopt_internal (argc, argv, optstring, longopts, longind, long_only)
-     int argc;
-     char *const *argv;
-     const char *optstring;
-     const struct option *longopts;
-     int *longind;
-     int long_only;
+_getopt_internal (int argc, char *const *argv, const char *optstring,
+		  const struct option *longopts, int *longind, int long_only)
 {
   int option_index;
 
@@ -433,8 +419,8 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
       const struct option *pfound = NULL;
       int indfound;
 
-      indfound = 0;             /* To silence the compiler. */
-      
+      indfound = 0;             /* To silence the compiler.  */
+
       while (*s && *s != '=')
         s++;
 
@@ -465,8 +451,8 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
       if (ambig && !exact)
         {
           if (opterr)
-            fprintf (stderr, "%s: option `%s' is ambiguous\n",
-                     argv[0], argv[optind]);
+            fprintf (stderr, _("%s: option `%s' is ambiguous\n"),
+                     exec_name, argv[optind]);
           nextchar += strlen (nextchar);
           optind++;
           return '?';
@@ -489,13 +475,13 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
                       if (argv[optind - 1][1] == '-')
                         /* --option */
                         fprintf (stderr,
-                                 "%s: option `--%s' doesn't allow an argument\n",
-                                 argv[0], pfound->name);
+                                 _("%s: option `--%s' doesn't allow an argument\n"),
+                                 exec_name, pfound->name);
                       else
                         /* +option or -option */
                         fprintf (stderr,
-                             "%s: option `%c%s' doesn't allow an argument\n",
-                             argv[0], argv[optind - 1][0], pfound->name);
+				 _("%s: option `%c%s' doesn't allow an argument\n"),
+				 exec_name, argv[optind - 1][0], pfound->name);
                     }
                   nextchar += strlen (nextchar);
                   return '?';
@@ -508,8 +494,9 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
               else
                 {
                   if (opterr)
-                    fprintf (stderr, "%s: option `%s' requires an argument\n",
-                             argv[0], argv[optind - 1]);
+                    fprintf (stderr,
+			     _("%s: option `%s' requires an argument\n"),
+                             exec_name, argv[optind - 1]);
                   nextchar += strlen (nextchar);
                   return optstring[0] == ':' ? ':' : '?';
                 }
@@ -538,12 +525,12 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
             {
               if (argv[optind][1] == '-')
                 /* --option */
-                fprintf (stderr, "%s: unrecognized option `--%s'\n",
-                         argv[0], nextchar);
+                fprintf (stderr, _("%s: unrecognized option `--%s'\n"),
+                         exec_name, nextchar);
               else
                 /* +option or -option */
-                fprintf (stderr, "%s: unrecognized option `%c%s'\n",
-                         argv[0], argv[optind][0], nextchar);
+                fprintf (stderr, _("%s: unrecognized option `%c%s'\n"),
+                         exec_name, argv[optind][0], nextchar);
             }
           nextchar = (char *) "";
           optind++;
@@ -568,12 +555,12 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 #if 0
             if (c < 040 || c >= 0177)
               fprintf (stderr, "%s: unrecognized option, character code 0%o\n",
-                       argv[0], c);
+                       exec_name, c);
             else
-              fprintf (stderr, "%s: unrecognized option `-%c'\n", argv[0], c);
+              fprintf (stderr, "%s: unrecognized option `-%c'\n", exec_name, c);
 #else
             /* 1003.2 specifies the format of this message.  */
-            fprintf (stderr, "%s: illegal option -- %c\n", argv[0], c);
+            fprintf (stderr, _("%s: illegal option -- %c\n"), exec_name, c);
 #endif
           }
         optopt = c;
@@ -609,11 +596,11 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
                   {
 #if 0
                     fprintf (stderr, "%s: option `-%c' requires an argument\n",
-                             argv[0], c);
+                             exec_name, c);
 #else
                     /* 1003.2 specifies the format of this message.  */
-                    fprintf (stderr, "%s: option requires an argument -- %c\n",
-                             argv[0], c);
+                    fprintf (stderr, _("%s: option requires an argument -- %c\n"),
+                             exec_name, c);
 #endif
                   }
                 optopt = c;
@@ -633,23 +620,16 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
   }
 }
 
-/* Calls internal getopt function to enable long option names. */
+/* Calls internal getopt function to enable long option names.  */
 int
-getopt_long (argc, argv, shortopts, longopts, longind)
-  int argc;
-  char *const *argv;
-  const char *shortopts;
-  const struct option *longopts;
-  int *longind;
+getopt_long (int argc, char *const *argv, const char *shortopts,
+	     const struct option *longopts, int *longind)
 {
   return _getopt_internal (argc, argv, shortopts, longopts, longind, 0);
 }
 
 int
-getopt (argc, argv, optstring)
-     int argc;
-     char *const *argv;
-     const char *optstring;
+getopt (int argc, char *const *argv, const char *optstring)
 {
   return _getopt_internal (argc, argv, optstring,
                            (const struct option *) 0,
